/*
 * Svjatoslav Commons - shared library of common functionality. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.commons.string.tokenizer;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Terminator {

    String regexp;
    public final TerminationStrategy termination;
    public final String group;
    public boolean active = true;
    public final Pattern pattern;

    public Terminator(TerminationStrategy termination, String regexp, String group) {
        this.termination = termination;
        this.group = group;
        this.regexp = regexp;
        this.pattern = Pattern.compile("^"+regexp);
    }

    public Matcher match(String source, int index) {
        Matcher matcher = pattern.matcher(source);
        matcher.region(index, source.length());
        return matcher;
    }

    @Override
    public String toString() {
        return "Terminator{" +
                "regexp='" + regexp + '\'' +
                ", termination=" + termination +
                ", group='" + group + '\'' +
                ", active=" + active +
                '}';
    }

    public enum TerminationStrategy {
        /**
         * Preserve token that is identified within Terminator and return it for processing. For example when
         * building language parser, it could be used for statements that you want to capture.
         */
        PRESERVE,

        /**
         * While tokens that are marked by Terminator are identified, they are dropped and not returned for consumption.
         * For example, when building language parser, you might use such strategy for whitespace and comments.
         * That is, those tokens act as separators between actually useful tokens, but you don't want to consume such
         * separators or comments in your code.
         */
        DROP
    }
}
