package eu.svjatoslav.commons.string.tokenizer;

import org.junit.Test;

import static eu.svjatoslav.commons.string.tokenizer.Terminator.TerminationStrategy.DROP;
import static eu.svjatoslav.commons.string.tokenizer.Terminator.TerminationStrategy.PRESERVE;
import static org.junit.Assert.*;

public class TokenizerTest {

    @Test
    public void testPeeking() throws Exception {
        Tokenizer tokenizer = new Tokenizer("this is a N'2015-03-18 09:48:54.360' test");
        tokenizer.addTerminator(DROP, "\\s");
        tokenizer.addTerminator(PRESERVE, "N'.*'");

        tokenizer.expectAndConsumeNextStringToken("this");

        assertEquals("is", tokenizer.peekNextToken().token);

        assertEquals("is", tokenizer.peekNextToken().token);

        assertTrue(tokenizer.peekIsOneOf("maybe", "is", "that"));
    }

    @Test
    public void testTokenization() throws Exception {
        Tokenizer tokenizer = new Tokenizer("\"hello world\"   /** comment **/   ((  is a N'2015-03-18 09:48:54.360' test");
        tokenizer.addTerminator(DROP,"\\s");
        tokenizer.addTerminator(PRESERVE,"\\(");
        tokenizer.addTerminator(PRESERVE, "\\\".*\\\"");
        tokenizer.addTerminator(PRESERVE, "N'.*'");
        tokenizer.addTerminator(DROP,"/\\*.*\\*/");

        assertTokenEquals("\"hello world\"", tokenizer);
        assertTokenEquals("(", tokenizer);
        assertTokenEquals("(", tokenizer);
        assertTokenEquals("is", tokenizer);
        assertTokenEquals("a", tokenizer);
        assertTokenEquals("N'2015-03-18 09:48:54.360'", tokenizer);
        assertTokenEquals("test", tokenizer);

        assertNull(tokenizer.getNextToken());
        assertFalse(tokenizer.hasMoreContent());
    }


    @Test
    public void testMultilineTokenization() throws Exception {
        Tokenizer tokenizer = new Tokenizer(
                "* heading\r\nnormal text\r\nnormal text 2");
        tokenizer.addTerminator(PRESERVE,"\\*[ \\t]+.*\\r?\\n");
        tokenizer.addTerminator(DROP,"\\r?\\n", "normalText");

        assertTokenEquals("* heading\r\n", tokenizer);
        assertTokenEquals("normal text", tokenizer);
        assertTokenEquals("normal text 2", tokenizer);

        assertNull(tokenizer.getNextToken());
        assertFalse(tokenizer.hasMoreContent());
    }


    private void assertTokenEquals(String expectedValue, Tokenizer tokenizer){
        assertEquals(expectedValue, tokenizer.getNextToken().token);
    }

}